#include "mqtt.hpp"

TaskHandle_t mqttTaskHandle = NULL;

WiFiClient wifiClient;
PubSubClient client(wifiClient);

void onMqttCallback(char* topic, byte* payload, unsigned int length)
{
    Serial.println("MQTT message arrived");
}

boolean connectMqtt()
{
    boolean result = client.connect("btclockClient");
    if (!result)
    {
        Serial.println("[MQTT] could not connect");
    }
    return result;
}

void setupMqtt()
{
    const String host = preferences.getString("mqttHost", "");
    if (host == "")
    {
        Serial.println("[MQTT] host not set");
        return;
    }
    Serial.print("[MQTT] host: ");
    Serial.println(host.c_str());

    IPAddress addr((uint32_t)0);
    if(!WiFi.hostByName(host.c_str(), addr))
    {
        Serial.println("[MQTT] host lookup fail");
        return;
    }
    client.setServer(addr, 1883);
    client.setCallback(onMqttCallback);

    connectMqtt();
}

// avoid circular deps, just forward declare externs used here.
bool hasLightLevel();
float getLightLevel();
String getMyHostname();

void mqttTask(void *pvParameters)
{
    int t=0;
    while (1)
    {
        client.loop();
        vTaskDelay(pdMS_TO_TICKS(1000));
        if (t++ % 10 == 0)
        {
            if (!client.connected())
            {
                // reconnect
                if (!connectMqtt())
                    return;
            }

            if (hasLightLevel())
            {
                std::string lux_s = std::to_string(static_cast<int>(std::round(getLightLevel())));
                publish("sensors/lux", lux_s.c_str());
            }
            if (WiFi.isConnected())
            {
                int8_t rssi = WiFi.RSSI();
                std::string rssi_s = std::to_string(static_cast<int>(rssi));
                publish("wifi/rssi", rssi_s.c_str());
            }
            std::string heap_free_s = std::to_string(static_cast<int>(ESP.getFreeHeap()));
            publish("mem/heap_free", heap_free_s.c_str());
            std::string heap_size_s = std::to_string(static_cast<int>(ESP.getHeapSize()));
            publish("mem/heap_size", heap_size_s.c_str());
        }
    }
}

void setupMqttTask()
{
    xTaskCreate(mqttTask, "mqttTask", 8192, NULL, 10, &mqttTaskHandle);
}

void publish(const char *topic, const char *value)
{
    if (!client.connected())
    {
        Serial.println("[MQTT] not connected");
        return;
    }

    const String hostname = getMyHostname();
    const String rootTopic = preferences.getString("mqttRootTopic", DEFAULT_MQTT_ROOTTOPIC);
    String fullTopic = rootTopic;
    if (!rootTopic.endsWith("/") && rootTopic != "")
    {
        fullTopic += "/";
    }
    fullTopic += hostname + "/" + topic;

    if (!client.publish(fullTopic.c_str(), value))
    {
        Serial.println("[MQTT] could not write");
    }
}